//
//  AudioViewController.m
//  Mac N-Back
//
//  Created by Grant Limberg on 10/28/08.
//  Copyright 2008 GL Software. All rights reserved.
//

#import "AudioViewController.h"
#import "PreferencesController.h"

@implementation AudioViewController

- (id) init {
	debug(@"%s:%d",__FUNCTION__,__LINE__);
	if(![super initWithNibName:@"AudioView" bundle:nil]) {
		return nil;
	}
	[self setTitle:@"Audio Options"];
	synth = [[NSSpeechSynthesizer alloc] init];
	voices = [NSSpeechSynthesizer availableVoices];
	return self;
}

#pragma mark Table Delegate Methods
- (NSInteger)numberOfRowsInTableView:(NSTableView *)aTableView {
	return [voices count];
}

-(id)tableView:(NSTableView*)tv objectValueForTableColumn:(NSTableColumn*)tableColumn row:(NSInteger)row {
	NSString *v = [voices objectAtIndex:row];
	NSDictionary *dict = [NSSpeechSynthesizer attributesForVoice:v];
	return [dict objectForKey:NSVoiceName];;
}

-(void)tableViewSelectionDidChange:(NSNotification*)notification {
	int row = [table selectedRow];
	if(row == -1)
		return;
	
	// change user defaults voice to new voice
	NSString *selectedVoice = [voices objectAtIndex:row];
	[synth setVoice:selectedVoice];
	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
	[defaults setObject:selectedVoice forKey:MNBDefaultVoice];
	NSLog(@"New voice = %@", selectedVoice);
}

#pragma mark awakeFromNib
-(void)awakeFromNib {
	// when the table view appears on screen, the default voice should be selected
	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
	int defaultRow = [voices indexOfObject:[defaults objectForKey:MNBDefaultVoice]];
	[table selectRow:defaultRow byExtendingSelection:NO];
	[table scrollRowToVisible:defaultRow];
}

@end
