//
//  GameController.m
//  Mac N-Back
//
//  Created by Grant Limberg on 10/21/08.
//  Copyright 2008 GL Software. All rights reserved.
//

#import "MacNBackAppDelegate.h"
#import "GameController.h"
#import "GameCell.h"
#import "GameSelection.h"
#import "GameStatus.h"
#import "PreferencesController.h"
#include <stdlib.h>

@interface GameController (PrivateMethods)

- (void)initLetterArray;

@end

@implementation GameController (PrivateMethods)

- (void)initLetterArray {
	letterArray = [NSArray arrayWithObjects:@"c",@"h",@"k",@"l",@"q",@"r",@"s",@"t",nil];
}

@end

@implementation GameController

@synthesize nBack;
@synthesize cellArray;
@synthesize appDelegate;
@synthesize currentCell;
@synthesize currentLetter;

- (id)init {
	debug(@"%s:%d", __FUNCTION__, __LINE__);
	if(self = [super init]) {
		selectionArray = [[NSMutableArray alloc] init];
		[self initLetterArray];
		nBack = 1;
		numRight = 0;
		numWrong = 0;
	}
	return self;
}

- (id)initWithNBack:(NSUInteger)n {
	debug(@"%s:%d",__FUNCTION__,__LINE__);
	if(self = [super init]) {
		selectionArray = [[NSMutableArray alloc] init];
		[self initLetterArray];
		nBack = n;
		numRight = 0;
		numWrong = 0;
		synth = [[NSSpeechSynthesizer alloc] init];
	}
	return self;
}

- (void)stopGame {
	debug(@"%s:%d",__FUNCTION__,__LINE__);
	for(int i=0; i<9; i++) {
		GameCell *cell = [cellArray objectAtIndex:i];
		[cell deselect];
	}
}

- (void)selectRandomCellAndLetter {
	debug(@"%s:%d",__FUNCTION__,__LINE__);
	
	// seed the random number generator with time
	srandom([NSDate timeIntervalSinceReferenceDate]);
	
	// give every selection a 1/4 chance of matching a letter or cell
	if( ([selectionArray count] > nBack) && ( (random()%4) == 0 ) )
	{
		GameSelection *selection = [selectionArray objectAtIndex:([selectionArray count]-nBack)];
		int chance = random()%3;
		if( chance == 0) {
			// match nBack cell
			debug(@"Match %i-back cell",nBack);
			self.currentCell = [selection cellNumber];
			NSInteger letter = random()%[letterArray count];
			self.currentLetter = [letterArray objectAtIndex:letter];
		} else if (chance == 1) {
			// match nBack letter
			debug(@"Match %i-back letter",nBack);
			self.currentLetter = [selection letter];
			self.currentCell = random()%[cellArray count];
		} else {
			// match nBack letter and cell
			debug(@"Match %i-back cell and letter",nBack);
			self.currentLetter = [selection letter];
			self.currentCell = [selection cellNumber];
		}
	
	} else {
	
		// Select random cell & letter
		srandom([NSDate timeIntervalSinceReferenceDate]);
		self.currentCell = random()%[cellArray count];
		NSInteger letter = random()%[letterArray count];
		self.currentLetter = [letterArray objectAtIndex:letter];
	
	}
	
	[status addRandomLetter:self.currentLetter];
	[status addRandomPosition:self.currentCell];
}


- (void)matchPosition {
	debug(@"%s:%d", __FUNCTION__, __LINE__);
	[status addPositionMatchGuess];
}

- (void)matchLetter {
	debug(@"%s:%d", __FUNCTION__, __LINE__);
	[status addLetterMatchGuess];
}

- (void)main {
	debug(@"%s:%d",__FUNCTION__,__LINE__);
	
	// load the correct user selected voice
	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
	synth = [[NSSpeechSynthesizer alloc] initWithVoice:[defaults objectForKey:MNBDefaultVoice]];
	
	// initialize a new game
	status = [[GameStatus alloc] initWithNBack:nBack];
	
	// TODO: Update loop to use timer instead of raw count
	for(int i=0; ![self isCancelled] && i < 23; i++) {
		[status resetFlags];
		[self selectRandomCellAndLetter];
		
		GameCell *cell = [cellArray objectAtIndex:self.currentCell];
		// add cell and letter selection to the game array
		GameSelection *selection = [[GameSelection alloc] init];
		[selection setCellNumber:self.currentCell];
		[selection setLetter:self.currentLetter];
		[selectionArray addObject:selection];
		
		[synth startSpeakingString:self.currentLetter];
		
		[cell setSelected];
		
		// pause for 1 second while the cell is selected
		// TODO: Update for varible speed
		for(int i=0; i<10; i++) {
			[NSThread sleepForTimeInterval:0.1];
			if([self isCancelled]) {
				[self stopGame];
				return;
			}
		}
		
		[cell deselect];
		
		// pause for 1.5 seconds after the cell is deselected
		// TODO: Update for variable speed
		for(int j=0; j<15; j++)  {
			[NSThread sleepForTimeInterval:0.1];
			if([self isCancelled]) {
				[self stopGame];
				return;
			}
		}
		[status checkMissedCell:selection];
	}
	
	debug(@"Game Completed");
	[appDelegate gameFinished:status];
}

@end
