//
//  GameStatus.h
//  Mac N-Back
//
//  Created by Grant Limberg on 10/23/08.
//  Copyright 2008 GL Software. All rights reserved.
//

#import <Cocoa/Cocoa.h>

@class GameSelection;

@interface GameStatus : NSObject {
	NSUInteger nBack;
	
	// cells and letters randomly selected by the game
	NSMutableArray *cells;
	NSMutableArray *letters;
	
	// cell & position match guesses made by the user
	NSMutableArray *audioMatchGuesses;
	NSMutableArray *positionMatchGuesses;
	
	NSUInteger numAudioMatchGuesses;
	NSUInteger numPositionMatchGuesses;
	NSUInteger numCorrectAudioGuesses;
	NSUInteger numCorrectPositionGuesses;
	NSUInteger numMissedAudioGuesses;
	NSUInteger numMissedPositionGuesses;
	NSUInteger totalPossible;
	
	BOOL acceptAudioMatchInput;
	BOOL acceptPositionmatchInput;
	
	NSLock *randomLock;
	NSLock *guessLock;
}

@property (readonly) NSUInteger numAudioMatchGuesses;
@property (readonly) NSUInteger numPositionMatchGuesses;
@property (readonly) NSUInteger numCorrectAudioGuesses;
@property (readonly) NSUInteger numCorrectPositionGuesses;
@property (readonly) NSUInteger numMissedAudioGuesses;
@property (readonly) NSUInteger numMissedPositionGuesses;
@property (readonly) NSUInteger totalPossible;

- (id)initWithNBack:(NSUInteger)num;
- (void)addRandomPosition:(NSUInteger)cell;
- (void)addRandomLetter:(NSString*)letter;
- (void)addPositionMatchGuess;
- (void)addLetterMatchGuess;
- (void)checkMissedCell:(GameSelection*)selection;
- (void)resetFlags;
@end
