//
//  GameStatus.m
//  Mac N-Back
//
//  Created by Grant Limberg on 10/23/08.
//  Copyright 2008 GL Software. All rights reserved.
//

#import "GameStatus.h"
#import "GameSelection.h"

@implementation GameStatus

@synthesize numAudioMatchGuesses;
@synthesize numPositionMatchGuesses;
@synthesize numCorrectAudioGuesses;
@synthesize numCorrectPositionGuesses;
@synthesize numMissedAudioGuesses;
@synthesize numMissedPositionGuesses;
@synthesize totalPossible;

- (id)initWithNBack:(NSUInteger)num {
	self = [super init];
	if(self) {
		nBack = num;
		
		numAudioMatchGuesses = 0;
		numPositionMatchGuesses = 0;
		numCorrectAudioGuesses = 0;
		numCorrectPositionGuesses = 0;
		numMissedAudioGuesses = 0;
		numMissedPositionGuesses = 0;
		totalPossible = 0;
		
		letters = [[NSMutableArray alloc] init];
		cells = [[NSMutableArray alloc] init];
		
		audioMatchGuesses = [[NSMutableArray alloc] init];
		positionMatchGuesses = [[NSMutableArray alloc] init];
		
		randomLock = [[NSLock alloc] init];
		guessLock = [[NSLock alloc] init];
		
		acceptAudioMatchInput = YES;
		acceptPositionmatchInput = YES;
	}
	return self;
}

- (void)addRandomPosition:(NSUInteger)cell {
	debug(@"%s:%d", __FUNCTION__, __LINE__);
	[randomLock lock];
	[cells addObject:[NSNumber numberWithLong:cell]];
	if([cells count] > nBack && cell == [[cells objectAtIndex:[cells count]-1-nBack] intValue])
		++totalPossible;
	[randomLock unlock];
}

- (void)addRandomLetter:(NSString*)letter {
	debug(@"%s:%d", __FUNCTION__, __LINE__);
	[randomLock lock];
	[letters addObject:letter];
	if([letters count] > nBack && [letter isEqualToString:[letters objectAtIndex:[letters count]-1-nBack]])
		++totalPossible;
	[randomLock unlock];
}

- (void)addPositionMatchGuess {
	debug(@"%s:%d", __FUNCTION__, __LINE__);
	if(acceptPositionmatchInput) {
		[guessLock lock];
		++numPositionMatchGuesses;
		[positionMatchGuesses addObject:[cells lastObject]];
		if([cells count] > nBack && [[cells lastObject] intValue] == [[cells objectAtIndex:[cells count]-nBack-1] intValue]) {
			++numCorrectPositionGuesses;
			acceptPositionmatchInput = NO;
			debug(@"Correct Position Guess");
		} else if ([cells count] > nBack && [[cells lastObject] intValue] != [[cells objectAtIndex:[cells count]-nBack-1] intValue]) {
			debug(@"Incorrect Position Guess");
		}
		[guessLock unlock];
	}
}

- (void)addLetterMatchGuess {
	debug(@"%s:%d", __FUNCTION__, __LINE__);
	if(acceptAudioMatchInput) {
		[guessLock lock];
		++numAudioMatchGuesses;
		[audioMatchGuesses addObject:[letters lastObject]];
		if([letters count] > nBack && [[letters lastObject] isEqualToString:[letters objectAtIndex:[letters count]-nBack-1]]) {
			++numCorrectAudioGuesses;
			acceptAudioMatchInput = NO;
			debug(@"Correct Audio Guess");
		} else if([letters count] > nBack && ![[letters lastObject] isEqualToString:[letters objectAtIndex:[letters count]-nBack-1]]) {
			debug(@"Incorrect Audio Guess");
		}
		[guessLock unlock];
	}
}

- (void)checkMissedCell:(GameSelection*)selection {
	[guessLock lock];
	[randomLock lock];
	debug(@"%s:%d", __FUNCTION__, __LINE__);
	if(acceptAudioMatchInput) {
		if( [letters count] > nBack && [[letters objectAtIndex:[letters count]-nBack-1] isEqualTo:[selection letter]] ) {
			debug(@"Missed Audio Match");
			++numMissedAudioGuesses;
		}
	}
	
	if(acceptPositionmatchInput) {
		if( [cells count] > nBack && [[cells objectAtIndex:[cells count]-nBack-1] intValue] == [selection cellNumber] ) {
			debug(@"Missed Position Match");
			++numMissedPositionGuesses;
		}
	}
	[randomLock unlock];
	[guessLock unlock];
}

- (void)resetFlags {
	debug(@"%s:%d", __FUNCTION__, __LINE__);
	acceptAudioMatchInput = YES;
	acceptPositionmatchInput = YES;
}
@end
