//
//  MacNBackAppDelegate.m
//  Mac N-Back
//
//  Created by Grant Limberg on 10/21/08.
//  Copyright 2008 GL Software. All rights reserved.
//

#import "MacNBackAppDelegate.h"
#import "GameSummary.h"
#import "GameStatus.h"
#import "PreferencesController.h"

@interface MacNBackAppDelegate (PrivateMethods)

- (NSMutableArray*)initCellArray;

@end

@implementation MacNBackAppDelegate (PrivateMethods)

- (NSMutableArray*)initCellArray {
	debug(@"%s:%d", __FUNCTION__, __LINE__);
	return [NSArray arrayWithObjects:cell1,cell2,cell3,cell4,cell5,cell6,cell7,cell8,cell9,nil];
}

@end


@implementation MacNBackAppDelegate

+ (void)initialize {
	debug(@"%s:%d", __FUNCTION__, __LINE__);
	NSMutableDictionary *defaultValues = [NSMutableDictionary dictionary];
	
	// set the default voice to the default system voice
	[defaultValues setObject:[NSSpeechSynthesizer defaultVoice] forKey:MNBDefaultVoice];
	[defaultValues setObject:[NSNumber numberWithFloat:1.0] forKey:MNBDefaultSpeed];
	[defaultValues setObject:@"Dual" forKey:MNBDefaultMode];
	[defaultValues setObject:[NSNumber numberWithInt:2] forKey:MNBDefaultNBack];
	
	[[NSUserDefaults standardUserDefaults] registerDefaults:defaultValues];
}

- (id)init {
	debug(@"%s:%d", __FUNCTION__, __LINE__);
	if(self = [super init]) {
		nBack = [[[NSUserDefaults standardUserDefaults] objectForKey:MNBDefaultNBack] intValue];;
		mode = [[NSUserDefaults standardUserDefaults] objectForKey:MNBDefaultMode];
	}
	return self;
}

- (id)initWithCoder:(NSCoder *)coder {
	debug(@"%s:%d", __FUNCTION__, __LINE__);
	if(self = [super init]) {
		nBack = [[[NSUserDefaults standardUserDefaults] objectForKey:MNBDefaultNBack] intValue];
		mode = [[NSUserDefaults standardUserDefaults] objectForKey:MNBDefaultMode];
	}
	return self;
}

- (void)encodeWithCoder:(NSCoder *)coder {
	debug(@"%s:%d", __FUNCTION__, __LINE__);
	[coder encodeObject:arrayController forKey:@"arrayController"];
}

- (void)awakeFromNib {
	debug(@"%s:%d", __FUNCTION__, __LINE__);
	[stopButton setEnabled:NO];
	[startButton setEnabled:YES];
	[positionMatchButton setEnabled:NO];
	[audioMatchButton setEnabled:NO];
	gameCells = [self initCellArray];
	
	NSString *path = [[[NSApplication sharedApplication] delegate] applicationSupportFolder];
	NSString *fileName = [path stringByAppendingString:@"/scores.mnb"];
	NSArray *tmpArray = [NSKeyedUnarchiver unarchiveObjectWithFile:fileName];
	for(GameSummary *g in tmpArray) {
		debug(@"%@-%f", [g gameMode], [g gameScore]);
		[arrayController addObject:g];
	}
}

- (IBAction)startGame:(id)sender {
	debug(@"%s:%d", __FUNCTION__, __LINE__);
	[startButton setEnabled:NO];
	[stopButton setEnabled:YES];
	
	[positionMatchButton setEnabled:YES];
	[audioMatchButton setEnabled:YES];
	
	// start game thread
	controller = [[GameController alloc] initWithNBack:nBack];
	[controller setAppDelegate:self];
	[controller setCellArray:gameCells];
	[controller start];
}

- (IBAction)stopGame:(id)sender {
	debug(@"%s:%d", __FUNCTION__, __LINE__);
	// stop game thread
	[controller cancel];	
	[startButton setEnabled:YES];
	[stopButton setEnabled:NO];
	[positionMatchButton setEnabled:NO];
	[audioMatchButton setEnabled:NO];
	controller = nil;
}

- (void)gameFinished:(GameStatus*)status {
	debug(@"%s:%d", __FUNCTION__, __LINE__);
	[startButton setEnabled:YES];
	[stopButton setEnabled:NO];
	controller = nil;
	
	[positionMatchButton setEnabled:NO];
	[audioMatchButton setEnabled:NO];
	
	// process scores & statistics
	double totalCorrect = [status numCorrectAudioGuesses] + [status numCorrectPositionGuesses];
	double totalGuesses = [status numAudioMatchGuesses] + [status numPositionMatchGuesses];
	double totalPossible = [status totalPossible];
	
	double score = (totalCorrect/(totalPossible+abs(totalGuesses-totalCorrect))*100);
	
	debug(@"totalGuesses: %f", totalGuesses);
	debug(@"totalCorrect: %f, totalPossible: %f, score: %f", totalCorrect, totalPossible, score);
	GameSummary *summary = [[GameSummary alloc] initWithMode:[NSString stringWithFormat:@"%@ %i-Back",mode,nBack] andScore:score];
	[arrayController addObject:summary];
	
#pragma mark Save Scores at End of Round
	// save scores
	NSFileManager *fileManager = [NSFileManager defaultManager];
	NSString *path = [[[NSApplication sharedApplication] delegate] applicationSupportFolder];
	
    if ( ![fileManager fileExistsAtPath:path isDirectory:NULL] ) {
        [fileManager createDirectoryAtPath:path attributes:nil];
    }
	
	NSString *fileName = [path stringByAppendingString:@"/scores.mnb"];	
	NSArray *tmpArray = [arrayController arrangedObjects];
	debug(@"Writing scores to %@", fileName);
	if([NSKeyedArchiver archiveRootObject:tmpArray toFile:fileName]) {
		debug(@"file written sucessfully");
	} else {
		debug(@"file not written");
	}
}

- (NSString *)applicationSupportFolder {
	debug(@"%s:%d", __FUNCTION__, __LINE__);
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSApplicationSupportDirectory, NSUserDomainMask, YES);
    NSString *basePath = ([paths count] > 0) ? [paths objectAtIndex:0] : NSTemporaryDirectory();
    return [basePath stringByAppendingPathComponent:@"Mac N Back"];
}

- (IBAction)audioMatch:(id)sender {
	debug(@"%s:%d", __FUNCTION__, __LINE__);
	[controller matchLetter];
}

- (IBAction)positionMatch:(id)sender {
	debug(@"%s:%d", __FUNCTION__, __LINE__);
	[controller matchPosition];
}

- (IBAction)showPreferencePanel:(id)sender {
	debug(@"%s:%d", __FUNCTION__, __LINE__);
	if(!preferencesController)
		preferencesController = [[PreferencesController alloc] init];
	[preferencesController showWindow:self];
}
@end
