//
//  PreferencesController.m
//  Mac N-Back
//
//  Created by Grant Limberg on 10/26/08.
//  Copyright 2008 GL Software. All rights reserved.
//

#import "PreferencesController.h"

NSString * const MNBDefaultVoice = @"DefaultVoice";
NSString * const MNBDefaultSpeed = @"DefaultSpeed";
NSString * const MNBDefaultMode = @"DefaultMode";
NSString * const MNBScores = @"Scores";
NSString * const MNBDefaultNBack = @"DefaultNBack";

@implementation PreferencesController

- (id)init {
	debug(@"%s:%d",__FUNCTION__,__LINE__);
	if(![super initWithWindowNibName:@"PreferencesPane"])
		return nil;
	
	viewControllers = [[NSMutableArray alloc] init];
	
	// Add general view controller
	ManagingViewController *vc;
	vc = [[GeneralViewController alloc] init];
	[viewControllers addObject:vc];
	
	// add audio view controller
	vc = [[AudioViewController alloc] init];
	[viewControllers addObject:vc];
	
	// add Updates view controller
	vc = [[UpdateViewController alloc] init];
	[viewControllers addObject:vc];
	
	return self;
}

- (IBAction)changeViewController:(id)sender {
	debug(@"%s:%d",__FUNCTION__,__LINE__);
	int i = [sender tag];
	ManagingViewController *vc = [viewControllers objectAtIndex:i];
	[self displayViewController:vc];
}

- (void)displayViewController:(ManagingViewController*)vc {
	debug(@"%s:%d",__FUNCTION__,__LINE__);
	// try to end editing
	NSWindow *w = [box window];
	BOOL ended = [w makeFirstResponder:w];
	if(!ended) {
		NSBeep();
		return;
	}
	NSView *v = [vc view];
	
	// compute the new window frame size
	NSSize currentSize = [[box contentView] frame].size;
	NSSize newSize = [v frame].size;
	float deltaWidth = newSize.width - currentSize.width;
	float deltaHeight = newSize.height - currentSize.height;
	NSRect windowFrame = [w frame];
	windowFrame.size.height += deltaHeight;
	windowFrame.size.width += deltaWidth;
	windowFrame.origin.y -= deltaHeight;
	
	// clear the box for resizing
	[box setContentView:nil];
	[w setFrame:windowFrame display:YES animate:YES];
	
	// put the view in the box
	[box setContentView:v];
}

- (void)windowDidLoad {
	debug(@"%s:%d",__FUNCTION__,__LINE__);
	[super windowDidLoad];
	NSMenu *menu = [popUp menu];
	int itemCount = itemCount = [viewControllers count];
	
	for(int i=0; i<itemCount; i++) {
		NSViewController *vc = [viewControllers objectAtIndex:i];
		NSMenuItem *mi = [[NSMenuItem alloc] initWithTitle:[vc title] 
																								action:@selector(changeViewController:) 
																				 keyEquivalent:@""];
		[mi setEnabled:YES];
		[mi setTag:i];
		[menu addItem:mi];
	}
	
	[self displayViewController:[viewControllers objectAtIndex:0]];
	[popUp selectItemAtIndex:0];
}


@end
